/*
 *	I2C_core.c
 *
 *	Created on: 2015/04/30
 *		Author: jinso(dung)
 */

/* Include file */
#include "../inc/I2C-core.h"
#include "../../DRIVER/inc/driver-common.h"
#include "../../DRIVER/inc/MAX10SoM-I2C.h"

/****************************************************************
alt_u8 i2c_xfer
	This function write/read data to/from register of component
inputs
	device_name:Name of component
	addr       :is point to point which store register'address
	data       :is point to point which store data for write/read
	dir        :1 = read,0 = write
	nbyte      :Number of bytes
return value
	status of transmission process
	0     :ACK
	1     :NACK
	ENODEV:miss device name
*****************************************************************/
alt_u8 i2c_xfer(char *device_name, alt_u8 *addr, alt_u8 *data, alt_u8 dir, alt_u8 nbyte)
{
	alt_u8 i, j, match = 0;
	alt_u8 rt;
	alt_u8 direc = dir;
	static struct I2C_info *board_info = I2C_board_info;
	static struct I2C_dev_info *dev_info;

	for ( i = 0; i < ARRAY_SIZE( I2C_board_info ); i++) {
		dev_info = board_info->dev_info;
		for ( j = 0; j < ARRAY_SIZE( I2C_board_info->dev_info ); j++ ) {
			if (strcmp(device_name,dev_info[j].device_name) == 0) {
				match = 1;
				break;
			}
		}
		if ( match )
			break;
		++board_info;
	}

	if ( !match ){
		printf("\tNo found device\n");
		return ENODEV;
	}

	switch ( direc ) {
		case 0:
			/* Starting and send out slave address */
			rt = I2C_start( board_info, device_name, direc );
			I2C_write( board_info, *addr, 0);
			if ( board_info->dev_info[j].config == I2C_ADD16 ) {
				addr++;
				I2C_write( board_info, *addr, 0);
			}

			for( i=1; i <= nbyte; i++ ) {
				if(i == nbyte){
					rt = I2C_write( board_info, *data, 1);
				}else{
					I2C_write( board_info, *data, 0);
				}
				/* Data update */
				data++;
			}
			break;
		case 1:
			/* Starting and send out slave address */
			rt = I2C_start( board_info, device_name, 0 );
			I2C_write( board_info, *addr, 0);
			if ( board_info->dev_info[j].config == I2C_ADD16 ) {
				addr++;
				I2C_write( board_info, *addr, 0);
			}
			rt = I2C_start( board_info, device_name, direc);
			for( i=1; i <= nbyte; i++ )	{
				if(i == nbyte){
					*data = I2C_read( board_info, 1);
				}else{
					*data = I2C_read( board_info, 0);
				}
				data++;
			}
			break;
		default:
			printf("dir = 0 is write mode, 1 is read mode\n");
			break;
	}

	return rt;
}

/****************************************************************
alt_u8 i2c_speed
	This function change speed of i2c channel
inputs
	device:i2c channel
	speed :new speed for change
return value
	0     :if update speed successfully
	ENODEV:miss device name
*****************************************************************/
alt_u8 i2c_speed(char *device, alt_u32 speed)
{
	alt_u8 i,match = 0;
	static struct I2C_info *board_info = I2C_board_info;

	/* Select the i2c channel to change speed */
	for ( i = 0; i < ARRAY_SIZE( I2C_board_info ); i++) {
		if(strcmp(device,board_info->device) == 0){
			board_info->speed = speed;
			match = 1;
			break;
		}
		++board_info;
	}

	if (!match){
		printf("\tNo found device\n");
		return ENODEV;
	}

	/* Initialize the i2c channel witch new speed */
	I2C_init(board_info);

	return 0;
}
