/*
 * PWM_core.c
 *
 *  Created on: 2015/04/30
 *      Author: Jinso(Nui)
 */

/* Include file */
#include "../inc/PWM-core.h"
#include "../../DRIVER/inc/driver-common.h"
#include "../../DRIVER/inc/MAX10SoM-PWM.h"

/****************************************************************
void pwm_duty
	This function is called when setting percent duty of the channel PWM
inputs
	device:name of the channel PWM
	duty  :percent duty(0 - 100)
return value
	void
*****************************************************************/
void pwm_duty(char *device, alt_u8 duty){
	alt_u8 i;
	alt_u16 data = 0, pwm_duty = 0;
	struct PWM_info *board_info = PWM_board_info;

	for ( i = 0; i < ARRAY_SIZE( PWM_board_info ); i++) {
		if ( strcmp(device, board_info->device) == 0 )
			break;
		++board_info;
	}
	data = PWM_read(board_info, PWM_PER);
	if(duty <= 100){
		pwm_duty = (int)(data*duty/100);
		PWM_write(board_info, PWM_DUTY, pwm_duty);
	}
}

/****************************************************************
void pwm_start
	This function is called when setting start count the channel PWM
inputs
	device:name of the channel PWM
return value
	void
*****************************************************************/
void pwm_start(char *device){
	alt_u8 i;
	struct PWM_info *board_info = PWM_board_info;

	for ( i = 0; i < ARRAY_SIZE( PWM_board_info ); i++) {
		if ( strcmp(device, board_info->device) == 0 )
			break;
		++board_info;
	}
	alt_u16 control;
	control = PWM_read(board_info, PWM_CTR);
	/*Setting start count PWM (control register)*/
	PWM_write(board_info, PWM_CTR, control | CTR_CNT);
}

/****************************************************************
void pwm_stop
	This function is called when setting stop count the channel PWM
inputs
	device:name of the channel PWM
return value
	void
*****************************************************************/
void pwm_stop(char *device){
	alt_u8 i;
	struct PWM_info *board_info = PWM_board_info;

	for ( i = 0; i < ARRAY_SIZE( PWM_board_info ); i++) {
		if ( strcmp(device, board_info->device) == 0 )
			break;
		++board_info;
	}
	alt_u16 control;
	control = PWM_read(board_info, PWM_CTR);
	/*Setting stop count PWM (control register)*/
	PWM_write(board_info, PWM_CTR, control & (~CTR_CNT));
}

/****************************************************************
void pwm_speed
	This function is called when setting speed for the channel PWM
inputs
	device:name of the channel PWM
	speed :speed of the channel PWM
return value
	void
*****************************************************************/

void pwm_speed(char *device, alt_u32 speed)
{
	alt_u8 i;
	struct PWM_info *board_info = PWM_board_info;
	alt_u32 pwm_div = 0, pwm_per;

	for ( i = 0; i < ARRAY_SIZE( PWM_board_info ); i++) {
		if ( strcmp(device, board_info->device) == 0 )
			break;
		++board_info;
	}

	board_info->speed = speed;
	pwm_per = (int)(board_info->clk/speed) - 1;

	if ( pwm_per > 0x0000ffff ) {
		pwm_per &= 0x0000ffff;
		pwm_div = (int)(board_info->clk/pwm_per);
		pwm_div /= speed;
	}

	/*Setting clock of PWM (divisor register)*/
	IOWR(board_info->address, PWM_DIV, pwm_div);

	/*Setting the cycle of PWM (period register)*/
	IOWR(board_info->address, PWM_PER, pwm_per);
}
