/*
 * SPI-core.c
 *
 *  Created on: 2015/05/18
 *      Author: Hiep
 */

/* Include file */
#include "../inc/SPI-core.h"
#include "../../DRIVER/inc/driver-common.h"
#include "../../DRIVER/inc/MAX10SoM-SPI.h"

/****************************************************************
alt_u8 spi_speed()
	This function changes the transfer speed of each spi channel.
inputs
	device: Device file "/dev/spi_x" (x=0, 1, 2, 3)
	speed : New speed for update.
return value
	0     : if update speed successfully
	ENODEV: miss device name
*****************************************************************/
alt_u8 spi_speed(char *device, alt_u32 speed)
{
	alt_u8 i, match = 0;

	struct SPI_info *board_info = SPI_board_info;

	for ( i = 0; i < ARRAY_SIZE( SPI_board_info ); i++) {
		if (strcmp(device,board_info->device) == 0) {
			board_info->speed = speed;
			match = 1;
			break;
		}
		++board_info;
	}

	if (!match){
		printf("\tNo found device\n");
		return ENODEV;
	}

	SPI_init(board_info);

	return 0;
}

/****************************************************************
alt_u8 SPI_xfer()
	This function write/read data to/from register of component
inputs
	device_name: Name of component
	data       : is point to point which store data for write/read
	dir        : 1 is read mode, 0 is write mode
	nbyte      : Number of bytes transfer
return value
	status of transmission process
	0     : if data transfer successfully
	ENODEV: miss device name
*****************************************************************/
alt_u8 spi_xfer(char *device_name, alt_u8 *data, alt_u8 dir, alt_u16 nbyte)
{
	alt_u8 i,j, match = 0;
	alt_u8 mode = dir;
	struct SPI_info *board_info = SPI_board_info;
	struct SPI_dev_info *dev_info;
	alt_u8 size_board = ARRAY_SIZE( SPI_board_info );
	alt_u8 size_dev = ARRAY_SIZE( SPI_board_info->dev_info );

	for ( i = 0; i < size_board; i++) {
		dev_info = board_info->dev_info;
		for ( j = 0; j < size_dev; j++ ) {
			if (strcmp(device_name,dev_info[j].device_name) == 0) {
				match = 1;
				break;
			}
		}
		if(match)
			break;
		++board_info;
	}

	if (!match){
		printf("\tNo found device\n");
		return ENODEV;
	}

	switch ( mode ) {
		case 0:		/* Write Data mode */
			/* WRITE DATA */
			SPI_set_cs(board_info);
			SPI_write(board_info, data, nbyte);
			SPI_clear_cs(board_info);
			break;
		case 1:		/* Read Data mode */
			/* READ DATA */
			SPI_set_cs(board_info);
			SPI_read(board_info, data, nbyte);
			SPI_clear_cs(board_info);
			break;
		default:
			printf("\n Unknown mode");
			break;
	}

	return 0;
}

/****************************************************************
alt_u8 spi_config()
	Transfer Mode Setting and Update.
inputs
	config:
		CONFIG_BIT8 8bit,
		CONFIG_BIT16 16bit,
		CONFIG_MSBF MSB First,
		CONFIG_LSBF LSB First,
	device: Device file "/dev/spi_x"(x=0, 1, 2, 3)
return value
	0:		if update config successfully
	ENODEV: if update config fail.
*****************************************************************/
alt_u8 spi_config(char *device, alt_u8 config)
{
	return 0;
}
