/*
 *	UART_core.c
 *
 *	Created on: 2015/04/30
 *		Author: Jinso(Nui)
 */

/* Include file */
#include "../inc/UART-core.h"
#include "../../DRIVER/inc/driver-common.h"
#include "../../DRIVER/inc/MAX10SoM-UART.h"

/****************************************************************
alt_u8 uart_baud
	This function is used when user want to change baud rate
	bit of the channel UART
inputs
	device:name of the channel UART
	baud  :value of the baud rate
return value
	0     :if update baud rate successfully
	ENODEV:miss device name
*****************************************************************/
alt_u8 uart_baud(char *device, alt_u32 baud)
{
	alt_u8 i, match = 0;
	alt_u16 divisor;
	struct UART_info *board_info = UART_board_info;

	for ( i = 0; i < ARRAY_SIZE(UART_board_info); i++) {
		if ( strcmp(device, board_info->device) == 0 ){
			match = 1;
			break;
		}
		++board_info;
	}

	if (!match){
		printf("\tNo found device\n");
		return ENODEV;
	}

	divisor = (alt_u16)(ALT_CPU_FREQ/baud + 0.5);
	IOWR(board_info->address, UART_DIV, divisor);

	return 0;
}

/****************************************************************
void uart_config
	This function is used when user want to change parity
	bit, stop bit, data bits of the channel UART
inputs

return value
	void
*****************************************************************/
void uart_config(char *device, alt_u8 config)
{

}

/****************************************************************
char *uart_xfer
	This function is used to send data string
inputs
	device_name:device name
	data	   :data transmission
	type	   :write or read
	nbyte	   :number byte of the data transmission
return value
	0     : send/receive successfully
	ENODEV: miss device name
	ENODATA: No data
*****************************************************************/
char uart_xfer(char *device_name, char *data, alt_u8 type, alt_u8 nbyte)
{
	alt_u8 i, match = 0;
	struct UART_info *board_info = UART_board_info;
	struct UART_dev_info *dev_info;
	for ( i = 0; i < ARRAY_SIZE(UART_board_info); i++) {
		dev_info = board_info->dev_info;
		if ( strcmp(device_name, dev_info->device_name) == 0 ){
			match = 1;
			break;
		}
		++board_info;
	}

	if (!match){
		printf("\tNo found device\n");
		return ENODEV;
	}

	if(type == WR){
	/*send data*/
		for ( i = 0; i < nbyte; i++ ) {
			uart_write(board_info, data[i]);
		}
	}else if(type == RE){
	/*receive data*/
		if(board_info->fifo.write >= nbyte){
			for ( i = 0; i < nbyte; i++ ) {
				if(board_info->fifo.read != board_info->fifo.write)
				{
					data[i] = board_info->fifo.buff[board_info->fifo.read];
					board_info->fifo.read++;
					if(BUFFER_SIZE <= board_info->fifo.read)
					{
						board_info->fifo.read = 0;
					}
				}
			}
			board_info->fifo.write = 0;
			board_info->fifo.read = 0;
		}else{
			return ENODATA;
		}
	}
	return 0;
}
