/*
 *	MAX10SoM-I2C.h
 *
 *	Created on: 2015/04/30
 *		Author: jinso(dung)
 */

#ifndef _MAX10SOM_I2C_H_
#define _MAX10SOM_I2C_H_

#include <io.h>
#include "alt_types.h"
#include "../../config.h"
#include "../../int.h"
#include <string.h>

/* I2C REGISTER of MAX10SoM(controller,master) */
/* ADDRESS OFFSET */
#define I2C_PRERL		(0)
#define I2C_PRERH		(1)
#define I2C_CTR			(2)
#define	I2C_TXR			(3)
#define	I2C_RXR			(3)
#define I2C_CR			(4)
#define	I2C_SR			(4)

/* BIT DEFINE */
/* CTR register bit */
#define CTR_EN			(0x80)
#define CTR_IEN			(0x40)
/* CR register bit */
#define CR_STA			(0x80)
#define CR_STO			(0x40)
#define CR_RD			(0x20)
#define CR_WR			(0x10)
#define	CR_ACK			(0x08)
#define	CR_IACK			(0x01)
/* SR register bit */
#define SR_RxACk		(0x80)
#define SR_BUSY			(0x40)
#define SR_AL			(0x20)
#define SR_TIP			(0x02)
#define SR_IF			(0x01)

/* Acknowledge sign */
#define I2C_ACK			(0)
#define I2C_NACK		(1)

/* Transfer Mode */
#define I2C_ADD16		(1)

/* I2C device information */
struct I2C_dev_info {
	char *device_name;
	alt_u8 slave_addr;
	alt_u8 config;
};

struct I2C_info {
	char *device;
	alt_u32 address;
	alt_u32 clk;
	alt_u32 speed;
	struct irq_info irq;
	struct I2C_dev_info dev_info[CONFIG_DEV_I2C_NUM];
};

extern struct I2C_info I2C_board_info[CONFIG_I2C_NUM];

/* FUNCTION OF MAX10SoM I2C  */
extern void I2C_init(struct I2C_info *board_info);
extern alt_u8 I2C_start(struct I2C_info *board_info, char *device_name, alt_u8 dir);
extern alt_u8 I2C_read(struct I2C_info *board_info, alt_u8 last);
extern alt_u8 I2C_write(struct I2C_info *board_info, alt_u8 data, alt_u8 last);


#endif /* _MAX10SOM_I2C_H_ */
