/*
 * MAX10SoM-SPI.h
 *
 *  Created on: 2015/05/18
 *      Author: Hiep
 */

#ifndef MAX10SOM_SPI_H_
#define MAX10SOM_SPI_H_

#include <io.h>
#include "alt_types.h"
#include "../../config.h"
#include "../../int.h"

/* SPI Register offset Setting */
#define SPI_CONTROL_REG			0x00		/* SPI Control Register */
#define SPI_STATUS_REG			0x01		/* SPI Status Register */
#define SPI_DTR					0x02		/* SPI Data Register */
#define SPI_RXDATA_REG			SPI_DTR		/* SPI Data Register */
#define SPI_TXDATA_REG			SPI_DTR		/* SPI Data Register */
#define SPI_BR_REG				0x03		/* SPI Bitrate Register */

/*SPI Control Register Bits Setting*/
#define SPI_CONTROL_INTE		(0x80)
#define SPI_CONTROL_CPOL		(0x04)
#define SPI_CONTROL_CPHA		(0x02)
#define SPI_CONTROL_MODE_MSK	(0x06)

/*SPI Status Register Bits Setting*/
#define SPI_STATUS_TXRDY		(0x01)
#define SPI_STATUS_TXEMP		(0x02)

#define SPI_MODE0	~(SPI_CONTROL_CPOL | SPI_CONTROL_CPHA) & SPI_CONTROL_MODE_MSK
#define SPI_MODE1	SPI_CONTROL_CPHA & SPI_CONTROL_MODE_MSK
#define SPI_MODE2	SPI_CONTROL_CPOL & SPI_CONTROL_MODE_MSK
#define SPI_MODE3	(SPI_CONTROL_CPOL | SPI_CONTROL_CPHA) & SPI_CONTROL_MODE_MSK

/*SPI DEVICES INFO*/
struct SPI_dev_info {
	char *device_name;					/* Slave Devices Name */
	alt_u8 pio_cs;						/* PIO Number of CS signal */
};

/*SPI INFO*/
struct SPI_info {
	char *device;						/* SPI Interface */
	alt_u32 address;					/* Base Address */
	alt_u32 clk;						/* Clock source */
	alt_u32 speed;						/* Transfer Speed */
	struct irq_info irq;				/* Interrupts */
	alt_u8 spi_mode;					/* SPI Transfer Mode */
	struct SPI_dev_info dev_info[1];	/* SPI Device Info */
};

extern struct SPI_info SPI_board_info[CONFIG_SPI_NUM];

/*SPI INITIALIZATION */
extern void SPI_init(struct SPI_info *board_info);
/*SPI WRITE FUNCTION */
extern alt_u8 SPI_write(struct SPI_info *board_info, const alt_u8 * write_data, alt_u16 write_length);
/*SPI READ FUNCTION */
extern alt_u8 SPI_read(struct SPI_info *board_info, alt_u8 * read_data, alt_u16 read_length );
/* SPI TRANSMISSION/RECEPTION CONDITION CHECK */
extern void spi_ready_wait(alt_u32 addr);
extern void SPI_clear_cs(struct SPI_info *board_info);
extern void SPI_set_cs(struct SPI_info *board_info);


#endif /* MAX10SOM_SPI_H_ */
