/*
 * MAX10SoM-UART.h
 *
 *  Created on: 2015/04/30
 *      Author: Jinso(Nui)
 */

#ifndef MAX10SOM_UART_H_
#define MAX10SOM_UART_H_

#include <io.h>
#include "system.h"
#include "alt_types.h"
#include "../../int.h"
#include <string.h>

/* UART REGISTER of MAX10SoM */
/* BASE ADDRESS */
#define UART0_BASE					UART_0_BASE
#define UART1_BASE					UART_1_BASE

/* ADDRESS OFFSET */
#define UART_RXR					(0)
#define UART_TXR					(1)
#define UART_STR					(2)
#define	UART_CTR					(3)
#define	UART_DIV					(4)

/* BIT DEFINE */
/* UART_STR register bit */
#define STR_PE						(0x01)
#define STR_FE						(0x02)
#define STR_BRK						(0x04)
#define STR_ROE						(0x08)
#define STR_TOE						(0x10)
#define STR_TMT						(0x20)
#define STR_TRDY					(0x40)
#define STR_RRDY					(0x80)
#define STR_E						(0x100)
#define STR_DCTS					(0x400)
#define STR_CTS						(0x800)
#define STR_EOP						(0x1000)

/* UART_CTR register bit */
#define CTR_PE						(0x01)
#define CTR_FE						(0x02)
#define CTR_BRK						(0x04)
#define CTR_ROE						(0x08)
#define CTR_TOE						(0x10)
#define CTR_TMT						(0x20)
#define CTR_TRDY					(0x40)
#define CTR_RRDY					(0x80)
#define CTR_E						(0x100)
#define CTR_DCTS					(0x400)
#define CTR_CTS						(0x800)
#define CTR_EOP						(0x1000)

/* PARAMETER DEFINE */
#define BUFFER_SIZE					(256)

/* UART device information */
struct UART_dev_info {
	char *device_name;
};

/* Structure UART FIFO*/
struct UART_FIFO
{
	alt_u16 read;
	alt_u16 write;
	unsigned char buff[BUFFER_SIZE];
};

struct UART_info {
	char *device;
	alt_u32 address;
	alt_u32 clk;
	alt_u32 baud;
	struct irq_info irq;
	struct UART_dev_info dev_info[1];
	struct UART_FIFO fifo;
};

extern struct UART_info UART_board_info[2];

/* FUNCTION OF MAX10SoM UART  */
void uart_init(struct UART_info *board_info);
void uart_write(struct UART_info *board_info, alt_u8 data);
alt_u8 uart_read(struct UART_info *board_info);
void uart0_irq_event(void* context, alt_u32 id);
void uart1_irq_event(void* context, alt_u32 id);

void handler_interrupt(struct UART_info *board_info);
void uart_rx_fifo(struct UART_info *board_info);


#endif /* MAX10SOM_UART_H_ */
