/*
 * MAX10SoM-PWM.c
 *
 *  Created on: 2015/04/30
 *      Author: Jinso(Nui)
 */

/* Include file */
#include "../inc/driver-common.h"
#include "../inc/MAX10SoM-PWM.h"

/****************************************************************
void PWM_init
	This function is called when initialize board.
	It is used to initialize PWM device
inputs
	board_info:information of the device using PWM
return value
	void
*****************************************************************/
void PWM_init(struct PWM_info *board_info)
{
	alt_u32 speed;
	alt_u32 pwm_div = 0, pwm_per;
	speed = board_info->speed;
#ifdef  DRI_DEBUG
	printf(" Initializing  PWM at 0x%x, \n\tClock source 0x%x \n\tSpeed 0x%x \n",board_info->address,board_info->clk,speed);
#endif
	pwm_per = (int)(board_info->clk/speed) - 1;

	if ( pwm_per > 0x0000ffff ) {
		pwm_per &= 0x0000ffff;
		pwm_div = (int)(board_info->clk/pwm_per);
		pwm_div /= speed;
	}

	/*clear control register*/
	IOWR(board_info->address, PWM_CTR, 0x0000);

	/*Setting out put PWM enable (control register)*/
	IOWR(board_info->address, PWM_CTR, CTR_OUT);
#if 0
	/*Setting clock of PWM (divisor register)*/
	IOWR(board_info->address, PWM_DIV, pwm_div);

	/*Setting the cycle of PWM (period register)*/
	IOWR(board_info->address, PWM_PER, 0x0200);
#else
	/*Setting clock of PWM (divisor register)*/
	IOWR(board_info->address, PWM_DIV, pwm_div);

	/*Setting the cycle of PWM (period register)*/
	IOWR(board_info->address, PWM_PER, pwm_per);
#endif
	/*Setting duty of the cycle of PWM(duty register)*/
	IOWR(board_info->address, PWM_DUTY, 0x0000);
}

/****************************************************************
unsigned long PWM_read
	The function reads out the value of of the component
inputs
	board_info:information of the device using PWM
	offset    :offset to target register
return value
	the value of target register
*****************************************************************/
alt_u16 PWM_read(struct PWM_info *board_info, alt_u8 offset)
{
	alt_u32 base_add;
	alt_u16 data;
	base_add = board_info->address;
	data = IORD(base_add, offset);
	return data;
}

/****************************************************************
void PWM_write
	assumes that any addressing and start
	has already been done.
	writes one byte of data to the base address
inputs
	board_info:information of the device using PWM
	offset    :offset to target register
	data      :byte to write
return value
	void
*****************************************************************/
void PWM_write(struct PWM_info *board_info, alt_u8 offset, alt_u16 data)
{
	alt_u32 base_add;
	base_add = board_info->address;
	IOWR(base_add, offset, data);
}
